import express from "express";
import { v4 as uuidv4 } from "uuid";
import dbPromise from "../config/database.js";
import { sendNtfyNotification } from "../services/notification.js";

const router = express.Router();

router.get("/", async (req, res) => {
	try {
		const db = await dbPromise;
		const forms = await db.all(`
			SELECT f.uuid, f.name, f.created_at, f.is_archived,
			(SELECT COUNT(*) FROM submissions WHERE form_uuid = f.uuid) as submission_count
			FROM forms f ORDER BY created_at DESC
		`);
		res.render("index", {
			forms,
			appUrl: `${req.protocol}://${req.get("host")}`,
		});
	} catch (error) {
		console.error("Error fetching forms:", error);
		res.status(500).send("Error fetching forms");
	}
});

router.post("/create-form", async (req, res) => {
	const formName = req.body.formName || "Untitled Form";
	const newUuid = uuidv4();
	try {
		const db = await dbPromise;
		await db.run("INSERT INTO forms (uuid, name) VALUES (?, ?)", [newUuid, formName]);
		console.log(`Form created: ${formName} with UUID: ${newUuid}`);
		await sendNtfyNotification(
			"New Form Created",
			`Form "${formName}" (UUID: ${newUuid}) was created.`,
			"high"
		);
		res.redirect("/admin");
	} catch (error) {
		console.error("Error creating form:", error);
		res.status(500).send("Error creating form");
	}
});

router.get("/submissions/:formUuid", async (req, res) => {
	const { formUuid } = req.params;
	const page = parseInt(req.query.page) || 1;
	const limit = parseInt(req.query.limit) || 10;
	const offset = (page - 1) * limit;

	try {
		const db = await dbPromise;
		const formDetails = await db.get("SELECT name FROM forms WHERE uuid = ?", [formUuid]);
		if (!formDetails) {
			return res.status(404).send("Form not found.");
		}

		// Get total count for pagination
		const countResult = await db.get(
			"SELECT COUNT(*) as total FROM submissions WHERE form_uuid = ?",
			[formUuid]
		);
		const totalSubmissions = countResult.total;
		const totalPages = Math.ceil(totalSubmissions / limit);

		const submissions = await db.all(
			"SELECT id, data, ip_address, submitted_at FROM submissions WHERE form_uuid = ? ORDER BY submitted_at DESC LIMIT ? OFFSET ?",
			[formUuid, limit, offset]
		);

		res.render("submissions", {
			submissions,
			formUuid,
			formName: formDetails.name,
			appUrl: `${req.protocol}://${req.get("host")}`,
			pagination: {
				currentPage: page,
				totalPages,
				totalSubmissions,
				limit,
			},
		});
	} catch (error) {
		console.error("Error fetching submissions:", error);
		res.status(500).send("Error fetching submissions");
	}
});

router.get("/submissions/:formUuid/export", async (req, res) => {
	const { formUuid } = req.params;
	try {
		const db = await dbPromise;
		const formDetails = await db.get("SELECT name FROM forms WHERE uuid = ?", [formUuid]);
		if (!formDetails) {
			return res.status(404).send("Form not found.");
		}

		const submissions = await db.all(
			"SELECT data, ip_address, submitted_at FROM submissions WHERE form_uuid = ? ORDER BY submitted_at DESC",
			[formUuid]
		);

		// Create CSV content
		const headers = ["Submitted At", "IP Address"];
		const rows = submissions.map((submission) => {
			const data = JSON.parse(submission.data);
			// Add all form fields as headers
			Object.keys(data).forEach((key) => {
				if (!headers.includes(key)) {
					headers.push(key);
				}
			});
			return {
				submitted_at: new Date(submission.submitted_at).toISOString(),
				ip_address: submission.ip_address,
				...data,
			};
		});

		// Generate CSV content
		let csvContent = headers.join(",") + "\n";
		rows.forEach((row) => {
			const values = headers.map((header) => {
				const value = row[header] || "";
				// Escape commas and quotes in values
				return `"${String(value).replace(/"/g, '""')}"`;
			});
			csvContent += values.join(",") + "\n";
		});

		// Set response headers for CSV download
		res.setHeader("Content-Type", "text/csv");
		res.setHeader(
			"Content-Disposition",
			`attachment; filename="${formDetails.name}-submissions.csv"`
		);
		res.send(csvContent);
	} catch (error) {
		console.error("Error exporting submissions:", error);
		res.status(500).send("Error exporting submissions");
	}
});

router.post("/delete-form/:formUuid", async (req, res) => {
	const { formUuid } = req.params;
	try {
		const db = await dbPromise;
		const formResult = await db.all("SELECT name FROM forms WHERE uuid = ?", [formUuid]);
		const formName =
			formResult.length > 0 ? formResult[0].name : `Form ${formUuid}`;

		await db.run("DELETE FROM forms WHERE uuid = ?", [formUuid]);
		console.log(`Form ${formUuid} deleted.`);
		await sendNtfyNotification(
			"Form Deleted",
			`Form "${formName}" (UUID: ${formUuid}) was deleted.`,
			"default"
		);
		res.redirect("/admin");
	} catch (error) {
		console.error("Error deleting form:", error);
		res.status(500).send("Error deleting form.");
	}
});

router.post("/delete-submission/:submissionId", async (req, res) => {
	const { submissionId } = req.params;
	let formUuidForRedirect = "unknown-form";
	try {
		const db = await dbPromise;
		const submissionResult = await db.all("SELECT form_uuid FROM submissions WHERE id = ?", [submissionId]);
		if (submissionResult.length > 0) {
			formUuidForRedirect = submissionResult[0].form_uuid;
		}

		await db.run("DELETE FROM submissions WHERE id = ?", [submissionId]);
		console.log(`Submission ${submissionId} deleted.`);
		await sendNtfyNotification(
			"Submission Deleted",
			`Submission ID ${submissionId} (for form ${formUuidForRedirect}) was deleted.`,
			"low"
		);
		if (formUuidForRedirect !== "unknown-form") {
			res.redirect(`/admin/submissions/${formUuidForRedirect}`);
		} else {
			res.redirect("/admin");
		}
	} catch (error) {
		console.error("Error deleting submission:", error);
		res.status(500).send("Error deleting submission.");
		if (formUuidForRedirect !== "unknown-form") {
			res.redirect(`/admin/submissions/${formUuidForRedirect}`);
		} else {
			res.redirect("/admin");
		}
	}
});

router.post("/update-form-name/:formUuid", async (req, res) => {
	const { formUuid } = req.params;
	const { newName } = req.body;

	if (!newName || newName.trim() === "") {
		return res.status(400).send("New form name is required.");
	}

	try {
		const db = await dbPromise;
		const formResult = await db.all("SELECT name FROM forms WHERE uuid = ?", [formUuid]);

		if (formResult.length === 0) {
			return res.status(404).send("Form not found.");
		}

		const oldName = formResult[0].name;

		await db.run("UPDATE forms SET name = ? WHERE uuid = ?", [
			newName,
			formUuid,
		]);

		console.log(
			`Form name updated from '${oldName}' to '${newName}' for UUID: ${formUuid}`
		);

		await sendNtfyNotification(
			"Form Name Updated",
			`Form name changed from '${oldName}' to '${newName}' for UUID: ${formUuid}.`,
			"default"
		);

		res.redirect("/admin");
	} catch (error) {
		console.error("Error updating form name:", error);
		res.status(500).send("Error updating form name.");
	}
});

router.post("/update-thank-you-url/:formUuid", async (req, res) => {
	const { formUuid } = req.params;
	const { thankYouUrl } = req.body;

	try {
		const db = await dbPromise;
		await db.run("UPDATE forms SET thank_you_url = ? WHERE uuid = ?", [thankYouUrl, formUuid]);
		console.log(`Thank You URL updated for form UUID: ${formUuid}`);
		res.redirect("/admin");
	} catch (error) {
		console.error("Error updating Thank You URL:", error);
		res.status(500).send("Error updating Thank You URL.");
	}
});

router.post("/update-ntfy-enabled/:formUuid", async (req, res) => {
	const { formUuid } = req.params;
	const ntfyEnabled = req.body.ntfyEnabled === "true";

	try {
		const db = await dbPromise;
		await db.run("UPDATE forms SET ntfy_enabled = ? WHERE uuid = ?", [
			ntfyEnabled,
			formUuid,
		]);
		console.log(`Ntfy notifications updated for form UUID: ${formUuid}`);
		res.redirect("/admin");
	} catch (error) {
		console.error("Error updating Ntfy notifications setting:", error);
		res.status(500).send("Error updating Ntfy notifications setting.");
	}
});

router.post("/clear-submissions/:formUuid", async (req, res) => {
	const { formUuid } = req.params;
	try {
		// First get form name for notification before clearing
		const db = await dbPromise;
		const formResult = await db.all("SELECT name FROM forms WHERE uuid = ?", [formUuid]);
		const formName =
			formResult.length > 0 ? formResult[0].name : `Form ${formUuid}`;

		// Delete all submissions for this form
		await db.run("DELETE FROM submissions WHERE form_uuid = ?", [formUuid]);
		console.log(`All submissions cleared for form ${formUuid}`);
		await sendNtfyNotification(
			"Submissions Cleared",
			`All submissions for form "${formName}" (UUID: ${formUuid}) were cleared.`,
			"default"
		);
		res.redirect(`/admin/submissions/${formUuid}`);
	} catch (error) {
		console.error("Error clearing submissions:", error);
		res.status(500).send("Error clearing submissions.");
	}
});

router.post("/update-thank-you-message/:formUuid", async (req, res) => {
	const { formUuid } = req.params;
	const { thankYouMessage } = req.body;
	try {
		const db = await dbPromise;
		await db.run("UPDATE forms SET thank_you_message = ? WHERE uuid = ?", [thankYouMessage, formUuid]);
		console.log(`Thank you message updated for form ${formUuid}`);
		res.redirect("/admin");
	} catch (error) {
		console.error("Error updating thank you message:", error);
		res.status(500).send("Error updating thank you message.");
	}
});

router.post("/archive-form/:formUuid", async (req, res) => {
	const { formUuid } = req.params;
	const { archive } = req.body;

	try {
		const db = await dbPromise;
		const formResult = await db.all("SELECT name FROM forms WHERE uuid = ?", [formUuid]);

		if (formResult.length === 0) {
			return res.status(404).send("Form not found.");
		}

		await db.run("UPDATE forms SET is_archived = ? WHERE uuid = ?", [
			archive === "true",
			formUuid,
		]);

		const action = archive === "true" ? "archived" : "unarchived";
		await sendNtfyNotification(
			`Form ${action}`,
			`Form "${formResult[0].name}" (UUID: ${formUuid}) has been ${action}.`,
			"default"
		);

		res.redirect("/admin");
	} catch (error) {
		console.error("Error updating form archive status:", error);
		res.status(500).send("Error updating form archive status");
	}
});

router.post("/update-allowed-domains/:formUuid", async (req, res) => {
	const { formUuid } = req.params;
	const { allowedDomains } = req.body;

	try {
		const db = await dbPromise;
		const formResult = await db.all("SELECT name FROM forms WHERE uuid = ?", [formUuid]);

		if (formResult.length === 0) {
			return res.status(404).send("Form not found.");
		}

		await db.run("UPDATE forms SET allowed_domains = ? WHERE uuid = ?", [
			allowedDomains,
			formUuid,
		]);

		await sendNtfyNotification(
			"Form Allowed Domains Updated",
			`Form "${formResult[0].name}" (UUID: ${formUuid}) allowed domains have been updated.`,
			"default"
		);

		res.redirect("/admin");
	} catch (error) {
		console.error("Error updating allowed domains:", error);
		res.status(500).send("Error updating allowed domains");
	}
});

router.post("/test-notification/:formUuid", async (req, res) => {
	const { formUuid } = req.params;

	try {
		const db = await dbPromise;
		const formResult = await db.all("SELECT name, ntfy_enabled FROM forms WHERE uuid = ?", [formUuid]);

		if (formResult.length === 0) {
			return res.status(404).send("Form not found.");
		}

		if (!formResult[0].ntfy_enabled) {
			return res
				.status(400)
				.send("Ntfy notifications are not enabled for this form.");
		}

		await sendNtfyNotification(
			"Test Notification",
			`This is a test notification for form "${formResult[0].name}" (UUID: ${formUuid}).`,
			"default",
			"test"
		);

		res.json({
			success: true,
			message: "Test notification sent successfully.",
		});
	} catch (error) {
		console.error("Error sending test notification:", error);
		res.status(500).send("Error sending test notification");
	}
});

export default router;
