import express from "express";
import dbPromise from "../config/database.js";
import { sendNtfyNotification } from "../services/notification.js";
import rateLimiter from "../middleware/rateLimiter.js";
import domainChecker from "../middleware/domainChecker.js";

const router = express.Router();

router.get("/", (req, res) => res.redirect("/admin"));
router.get("/health", (req, res) => res.status(200).json({ status: "ok" }));

router.post(
	"/submit/:formUuid",
	rateLimiter,
	domainChecker,
	async (req, res) => {
		const { formUuid } = req.params;
		const submissionData = { ...req.body };
		const ipAddress = req.ip;

		if (submissionData.honeypot_field && submissionData.honeypot_field !== "") {
			console.log(
				`Honeypot triggered for ${formUuid} by IP ${ipAddress}. Ignoring submission.`
			);
			if (submissionData._thankyou) {
				return res.redirect(submissionData._thankyou);
			}
			return res.send(
				"<h1>Thank You!</h1><p>Your submission has been received.</p>"
			);
		}
		delete submissionData.honeypot_field;

		let formNameForNotification = `Form ${formUuid}`;
		try {
			const db = await dbPromise;
			const form = await db.get(
				"SELECT id, name, thank_you_url, thank_you_message, ntfy_enabled, is_archived FROM forms WHERE uuid = ?",
				[formUuid]
			);
			if (!form) {
				return res.status(404).send("Form endpoint not found.");
			}

			if (form.is_archived) {
				return res
					.status(410)
					.send(
						"This form has been archived and is no longer accepting submissions."
					);
			}

			formNameForNotification = form.name || `Form ${formUuid}`;
			const ntfyEnabled = form.ntfy_enabled;

			await db.run(
				"INSERT INTO submissions (form_uuid, data, ip_address) VALUES (?, ?, ?)",
				[formUuid, JSON.stringify(submissionData), ipAddress]
			);
			console.log(`Submission received for ${formUuid}:`, submissionData);

			const submissionSummary = Object.entries(submissionData)
				.filter(([key]) => key !== "_thankyou")
				.map(([key, value]) => `${key}: ${value}`)
				.join(", ");

			if (ntfyEnabled) {
				await sendNtfyNotification(
					`New Submission: ${formNameForNotification}`,
					`Data: ${submissionSummary || "No data fields"
					}\nFrom IP: ${ipAddress}`,
					"high",
					"incoming_form"
				);
			}

			if (form.thank_you_url) {
				return res.redirect(form.thank_you_url);
			}

			if (form.thank_you_message) {
				return res.send(`<h1>Thank You!</h1><p>${form.thank_you_message}</p>`);
			}

			if (submissionData._thankyou) {
				return res.redirect(submissionData._thankyou);
			}

			res.send(
				'<h1>Thank You!</h1><p>Your submission has been received.</p><p><a href="/">Back to formies</a></p>'
			);
		} catch (error) {
			console.error("Error processing submission:", error);
			await sendNtfyNotification(
				`Submission Error: ${formNameForNotification}`,
				`Failed to process submission for ${formUuid} from IP ${ipAddress}. Error: ${error.message}`,
				"max"
			);
			res.status(500).send("Error processing submission.");
		}
	}
);

export default router;
